==> StanfordParser-1.3.tar.gz: lexparser-2003-09-06/src/edu/stanford/nlp/trees/NPTmpRetainingTreeNormalizer.java <==
package edu.stanford.nlp.trees;

/** Same TreeNormalizer as BobChrisTreeNormalizer, but optionally provides
 *  four extras.  I.e., the class name is now a misnomer.
 *  1) retains -TMP labels on NP with the new identification NP-TMP,
 *     and marks the tag which heads this with a ^TMP <br>
 *  2) Annotates S nodes which contain a gapped subject: i.e., 
 *     <code>S &lt; (/^NP-SBJ/ &lt; -NONE-) --> S-G</code>  <br>
 *  3) Leave all functional tags on nodes.
 *  4) Extended processing of -TMP modifiers
 *
 *  @author Christopher Manning
 *  @author Dan Klein
 */
public class NPTmpRetainingTreeNormalizer extends BobChrisTreeNormalizer {

  private final boolean doTemporalStuff;
  private final boolean doExtendedTemporalStuff;
  private final boolean doSGappedStuff;
  private final boolean leaveItAll;

    public NPTmpRetainingTreeNormalizer() {
	this(true, false);
    }

  public NPTmpRetainingTreeNormalizer(boolean doTemporalStuff,
				      boolean doSGappedStuff) {
    this(doTemporalStuff, false, doSGappedStuff, false);
  }

  public NPTmpRetainingTreeNormalizer(boolean doTemporalStuff,
				      boolean doExtendedTemporalStuff,
				      boolean doSGappedStuff,
				      boolean leaveItAll) {
    this.doTemporalStuff = doTemporalStuff;
    this.doExtendedTemporalStuff = doExtendedTemporalStuff;
    this.doSGappedStuff = doSGappedStuff;
    this.leaveItAll = leaveItAll;
  }

  private String stripNums(String label) {
    int loc = label.length()-1;
    while (Character.isDigit(label.charAt(loc)) ||
	   label.charAt(loc) == '-') {
      loc--;
    }
    return label.substring(0,loc+1);
  }


    /** Remove things like hyphened functional tags and equals from the
     *  end of a node label.
     */
    protected String cleanUpLabel(String label) {
	if (label == null) {
	    label = "ROOT";
	    // String constants are always interned
	} else {
	    if (leaveItAll) {
		return stripNums(label);
	    }
	    boolean temp = label.startsWith("NP-TMP");
	    boolean anytemp = label.matches(".*-TMP.*");
	    boolean subj = label.startsWith("NP-SBJ");
	    label = tlp.basicCategory(label);
	    if (doExtendedTemporalStuff && anytemp) {
		label += "-TMP";
	    } else if (doTemporalStuff && temp) {
		label = label + "-TMP";
	    }
	    if (doSGappedStuff && subj) {
		label = label + "-SBJ";
	    }
	}
	return label;
    }


    private static boolean includesEmptyNPSubj(Tree t) {
      if (t == null) return false;
      Tree[] kids = t.children();
      if (kids == null) return false;
      for (int i = 0; i < kids.length; i++) {
	Tree[] kidkids = kids[i].children();
	if (kids[i].label().value().startsWith("NP-SBJ")) {
	  kids[i].label().setValue("NP");
	  if (kidkids != null && kidkids.length == 1 &&
	      kidkids[0].label().value().equals("-NONE-")) {
	    return true;
	  }
	}
      }
      return false;
    }


    /** Normalize a whole tree -- one can assume that this is the root.
     *  This implementation deletes empty elements (ones with nonterminal
     *  tag label '-NONE-') from the tree.  It does work for a null tree.
     */
    public Tree normalizeWholeTree(Tree tree, TreeFactory tf) {
	final HeadFinder hf = new CollinsHeadFinder();
	return tree.transform(new TreeTransformer() {
	      public Tree transformTree(Tree t, TreeFactory tf) {
		if (doSGappedStuff) {
		  String lab = t.label().value();
		  if (lab.equals("S") && includesEmptyNPSubj(t)) {
		    LabelFactory lf = t.label().labelFactory();
		    // Note: this changes the tree label, rather than
		    // creating a new tree node.  Beware!
		    t.setLabel(lf.newLabel(t.label().value() + "-G"));
		  }
		}
		return t;
	      }
	    }, tf).prune(new edu.stanford.nlp.util.Filter() {
	    public boolean accept(Object obj) {
		Tree t = (Tree) obj;
		Tree[] kids = t.children();
		Label l = t.label();
		if ((l != null) && 
		    l.value() != null &&
		    (l.value().equals("-NONE-")) &&
		    ! t.isLeaf() && 
		    kids.length == 1 && kids[0].isLeaf()) {
		    // Delete empty/trace nodes (ones marked '-NONE-')
		    return false;
		}
		return true;
	    }
	}, tf).spliceOut(new edu.stanford.nlp.util.Filter() {
	    public boolean accept(Object obj) {
	      Tree t = (Tree) obj;
	      if (t.isLeaf() ||
		  t.isPreTerminal() ||
		  t.children().length != 1)
		return true;
	      if (t.label() != null &&
		  t.label().equals(t.children()[0].label()))
		return false;
	      return true;
	    }
	  }, tf).transform(new TreeTransformer() {
	      public Tree transformTree(Tree t, TreeFactory tf) {
		if (doExtendedTemporalStuff) {
		  String lab = t.label().value();
		  if (lab.matches(".*-TMP")) {
		    Tree ht = hf.determineHead(t);
		    while ( ! ht.isLeaf()) {
		      LabelFactory lf = ht.label().labelFactory();
		      // Note: this changes the tree label, rather than
		      // creating a new tree node.  Beware!
		      ht.setLabel(lf.newLabel(ht.label().value() + "-TMP"));
		      ht = hf.determineHead(ht);
		    }
		  }
		} else if (doTemporalStuff) {
		  String lab = t.label().value();
		  if (lab.equals("NP-TMP")) {
		    Tree ht = hf.determineHead(t);
		    while ( ! ht.isPreTerminal()) {
		      ht = hf.determineHead(ht);
		    }
		    LabelFactory lf = ht.label().labelFactory();
		    // Note: this changes the tree label, rather than
		    // creating a new tree node.  Beware!
		    ht.setLabel(lf.newLabel(ht.label().value() + "-TMP"));
		  }
		}
		return t;
	      }
	    }, tf);
    }

}

==> StanfordParser-1.4.tar.gz: lexparser-2004-03-24/src/edu/stanford/nlp/trees/NPTmpRetainingTreeNormalizer.java <==
package edu.stanford.nlp.trees;

import java.util.regex.*;

/** Same TreeNormalizer as BobChrisTreeNormalizer, but optionally provides
 *  five extras.  I.e., the class name is now a misnomer.<br>
 *  1) retains -TMP labels on NP with the new identification NP-TMP,
 *     and marks the tag which heads this with a ^TMP <br>
 *  2) Annotates S nodes which contain a gapped subject: i.e., 
 *     <code>S &lt; (/^NP-SBJ/ &lt; -NONE-) --> S-G</code>  <br>
 *  3) Leave all functional tags on nodes. <br>
 *  4) Extended processing of -TMP modifiers <br>
 *  5) Keeps -ADV labels on NP and marks head tag with &`^ADV
 *
 *  @author Christopher Manning
 *  @author Dan Klein
 */
public class NPTmpRetainingTreeNormalizer extends BobChrisTreeNormalizer {

  private final boolean doTemporalStuff;
  private final boolean doExtendedTemporalStuff;
  private final boolean doSGappedStuff;
  private final boolean leaveItAll;
  private final boolean doAdverbialNP;
  private final Pattern NPTmpPattern;
  private final Pattern TmpPattern;
  private final Pattern NPSbjPattern;
  private final Pattern NPAdvPattern;


  public NPTmpRetainingTreeNormalizer() {
    this(true, false);
  }

  public NPTmpRetainingTreeNormalizer(boolean doTemporalStuff,
				      boolean doSGappedStuff) {
    this(doTemporalStuff, false, doSGappedStuff, false, false);
  }

  public NPTmpRetainingTreeNormalizer(boolean doTemporalStuff,
				      boolean doExtendedTemporalStuff,
				      boolean doSGappedStuff,
				      boolean leaveItAll,
				      boolean doAdverbialNP) {
    this.doTemporalStuff = doTemporalStuff;
    this.doExtendedTemporalStuff = doExtendedTemporalStuff;
    this.doSGappedStuff = doSGappedStuff;
    this.leaveItAll = leaveItAll;
    this.doAdverbialNP = doAdverbialNP;
    NPTmpPattern = Pattern.compile("NP.*-TMP.*");
    TmpPattern = Pattern.compile(".*-TMP.*");
    NPAdvPattern = Pattern.compile("NP.*-ADV.*");
    NPSbjPattern = Pattern.compile("NP.*-SBJ.*");
  }


  private String stripNums(String label) {
    int loc = label.length()-1;
    while (Character.isDigit(label.charAt(loc)) ||
	   label.charAt(loc) == '-') {
      loc--;
    }
    return label.substring(0,loc+1);
  }


    /** Remove things like hyphened functional tags and equals from the
     *  end of a node label.
     */
    protected String cleanUpLabel(String label) {
	if (label == null) {
	    label = "ROOT";
	    // String constants are always interned
	} else {
	    if (leaveItAll) {
		return stripNums(label);
	    }
	    boolean temp = NPTmpPattern.matcher(label).matches();
	    boolean anytemp = TmpPattern.matcher(label).matches();
	    boolean subj = NPSbjPattern.matcher(label).matches();
	    boolean npadv = NPAdvPattern.matcher(label).matches();
	    label = tlp.basicCategory(label);
	    if (doExtendedTemporalStuff && anytemp) {
		label += "-TMP";
	    } else if (doTemporalStuff && temp) {
		label = label + "-TMP";
	    }
	    if (doAdverbialNP && npadv) {
	        label = label + "-ADV";
	    }
	    if (doSGappedStuff && subj) {
		label = label + "-SBJ";
	    }
	}
	return label;
    }


    private static boolean includesEmptyNPSubj(Tree t) {
      if (t == null) return false;
      Tree[] kids = t.children();
      if (kids == null) return false;
      for (int i = 0; i < kids.length; i++) {
	Tree[] kidkids = kids[i].children();
	if (kids[i].label().value().startsWith("NP-SBJ")) {
	  kids[i].label().setValue("NP");
	  if (kidkids != null && kidkids.length == 1 &&
	      kidkids[0].label().value().equals("-NONE-")) {
	    return true;
	  }
	}
      }
      return false;
    }


    /** Normalize a whole tree -- one can assume that this is the root.
     *  This implementation deletes empty elements (ones with nonterminal
     *  tag label '-NONE-') from the tree.
     */
    public Tree normalizeWholeTree(Tree tree, TreeFactory tf) {
	final HeadFinder hf = new CollinsHeadFinder();
	return tree.transform(new TreeTransformer() {
	      public Tree transformTree(Tree t, TreeFactory tf) {
		if (doSGappedStuff) {
		  String lab = t.label().value();
		  if (lab.equals("S") && includesEmptyNPSubj(t)) {
		    LabelFactory lf = t.label().labelFactory();
		    // Note: this changes the tree label, rather than
		    // creating a new tree node.  Beware!
		    t.setLabel(lf.newLabel(t.label().value() + "-G"));
		  }
		}
		return t;
	      }
	    }, tf).prune(new edu.stanford.nlp.util.Filter() {
	    public boolean accept(Object obj) {
		Tree t = (Tree) obj;
		Tree[] kids = t.children();
		Label l = t.label();
		if ((l != null) && 
		    l.value() != null &&
		    (l.value().equals("-NONE-")) &&
		    ! t.isLeaf() && 
		    kids.length == 1 && kids[0].isLeaf()) {
		    // Delete empty/trace nodes (ones marked '-NONE-')
		    return false;
		}
		return true;
	    }
	}, tf).spliceOut(new edu.stanford.nlp.util.Filter() {
	    public boolean accept(Object obj) {
	      Tree t = (Tree) obj;
	      if (t.isLeaf() ||
		  t.isPreTerminal() ||
		  t.children().length != 1)
		return true;
	      if (t.label() != null &&
		  t.label().equals(t.children()[0].label()))
		return false;
	      return true;
	    }
	  }, tf).transform(new TreeTransformer() {
	      public Tree transformTree(Tree t, TreeFactory tf) {
		if (doExtendedTemporalStuff) {
		  String lab = t.label().value();
		  if (lab.matches(".*-TMP")) {
		    Tree oldT = t;
		    Tree ht;
		    do {
		      ht = hf.determineHead(oldT);
		      // special fix for possessives! -- make noun before head
		      if (ht.label().value().equals("POS")) {
			int j = oldT.indexOf(ht);
			if (j > 0) {
			  ht = oldT.getChild(j - 1);
			}
		      }
		      LabelFactory lf = ht.label().labelFactory();
		      // Note: this changes the tree label, rather than
		      // creating a new tree node.  Beware!
		      ht.setLabel(lf.newLabel(ht.label().value() + "-TMP"));
		      oldT = ht;
		    } while ( ! ht.isPreTerminal());
		    if (lab.startsWith("PP")) {
		      ht = hf.determineHead(t);
		      // look to right
		      int j = t.indexOf(ht);
		      int sz = t.children().length;
		      if (j + 1 < sz) {
			ht = t.getChild(j + 1);
		      }
		      if (ht.label().value().startsWith("NP")) {
			while ( ! ht.isLeaf()) {
			  LabelFactory lf = ht.label().labelFactory();
			  // Note: this changes the tree label, rather than
			  // creating a new tree node.  Beware!
			  ht.setLabel(lf.newLabel(ht.label().value()+"-TMP"));
			  ht = hf.determineHead(ht);
			}
		      }
		    }
		  }
		} else if (doTemporalStuff) {
		  String lab = t.label().value();
		  if (lab.equals("NP-TMP")) {
		    Tree oldT = t;
		    Tree ht;
		    do {
		      ht = hf.determineHead(oldT);
		      // special fix for possessives! -- make noun before head
		      if (ht.label().value().equals("POS")) {
			int j = oldT.indexOf(ht);
			if (j > 0) {
			  ht = oldT.getChild(j - 1);
			}
		      }
		      oldT = ht;
		    } while ( ! ht.isPreTerminal());
		    LabelFactory lf = ht.label().labelFactory();
		    // Note: this changes the tree label, rather than
		    // creating a new tree node.  Beware!
		    ht.setLabel(lf.newLabel(ht.label().value() + "-TMP"));
		  }
		}
		if (doAdverbialNP) {
		  String lab = t.label().value();
		  if (lab.equals("NP-ADV")) {
		    Tree oldT = t;
		    Tree ht;
		    do {
		      ht = hf.determineHead(oldT);
		      // special fix for possessives! -- make noun before head
		      if (ht.label().value().equals("POS")) {
			int j = oldT.indexOf(ht);
			if (j > 0) {
			  ht = oldT.getChild(j - 1);
			}
		      }
		      oldT = ht;
		    } while ( ! ht.isPreTerminal());
		    LabelFactory lf = ht.label().labelFactory();
		    // Note: this changes the tree label, rather than
		    // creating a new tree node.  Beware!
		    ht.setLabel(lf.newLabel(ht.label().value() + "-ADV"));
		  }
		}
		return t;
	      }
	    }, tf);
    }

}
